## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup
import CvCustomEventManager

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer

class CustomFunctions:

    def __init__(self):
        self.cityInfrastructure = {}
        self.unitHasDiedThisTurn = {}

    def initCityProcess(self, pCity):
        pCityID = pCity.getID()

        self.cityInfrastructure[pCityID] = {}
        self.cityInfrastructure[pCityID]['WEALTH'] = 0
        self.cityInfrastructure[pCityID]['RESEARCH'] = 0
        self.cityInfrastructure[pCityID]['CULTURE'] = 0
        self.cityInfrastructure[pCityID]['ESPIONAGE'] = 0
        self.cityInfrastructure[pCityID]['PRODUCTION'] = 0

    def updateCityProcess(self, pCity):
        pCityID = pCity.getID()

        if pCity.getProductionProcess() == gc.getInfoTypeForString("PROCESS_WEALTH"):
            self.updateCityProcessBuildings(pCity, 'WEALTH')
        elif pCity.getProductionProcess() == gc.getInfoTypeForString("PROCESS_RESEARCH"):
            self.updateCityProcessBuildings(pCity, 'RESEARCH')
        elif pCity.getProductionProcess() == gc.getInfoTypeForString("PROCESS_CULTURE"):
            self.updateCityProcessBuildings(pCity, 'CULTURE')
        elif pCity.getProductionProcess() == gc.getInfoTypeForString("PROCESS_ESPIONAGE"):
            self.updateCityProcessBuildings(pCity, 'ESPIONAGE')
        elif pCity.getProductionProcess() == gc.getInfoTypeForString("PROCESS_PRODUCTION"):
            self.updateCityProcessBuildings(pCity, 'PRODUCTION')

    def updateCityProcessBuildings(self, pCity, processType):

#        maxpowerof2 = 8
        bonus = 0
        pCityID = pCity.getID()

#        pCity.changeFood(15)

        if not pCityID in self.cityInfrastructure:
            self.cityInfrastructure[pCityID] = {}
#       If loading, calculate the hammers that have been invested.
        if not processType in self.cityInfrastructure[pCityID]:
            if processType == 'PRODUCTION':
                self.cityInfrastructure[pCityID][processType] = int(pCity.getNumBuilding(gc.getInfoTypeForString("BUILDING_" + processType + "_BONUS")) ** (1.0/0.55))
            else:
                self.cityInfrastructure[pCityID][processType] = int(pCity.getNumBuilding(gc.getInfoTypeForString("BUILDING_" + processType + "_BONUS")) ** (1.0/0.7))
                                                                        
        self.cityInfrastructure[pCityID][processType] += pCity.getYieldRate(YieldTypes.YIELD_PRODUCTION)

        if processType == 'PRODUCTION':
            bonus = int((self.cityInfrastructure[pCityID][processType]) ** 0.55)
#            pCity.changeFood(self.cityInfrastructure[pCityID][processType])
        else:
            bonus = int((self.cityInfrastructure[pCityID][processType]/2) ** 0.7)
            
##      First, reset the buildings in case the bonus has changed
#        for i in range(maxpowerof2):
        pCity.setNumRealBuilding(gc.getInfoTypeForString("BUILDING_" + processType + "_BONUS"), 0)

#       Now, determine the bonus buildings based on the respective power of 2 that applies
#        for i in range(maxpowerof2):
#            n = (maxpowerof2 - 1) - i
#            if bonus >= 2**n:
#                pCity.setNumRealBuilding(gc.getInfoTypeForString("BUILDING_" + processType + "_BONUS_" + str(n)), 1)
#                bonus -= 2**n
        pCity.setNumRealBuilding(gc.getInfoTypeForString("BUILDING_" + processType + "_BONUS"), bonus)

    def saveGreatGeneral(self, iUnit, iPlayer, iAttacker):
        pCity = iPlayer.getCapitalCity()
        experience = iUnit.getExperience()
        iUnitID = iUnit.getID()
        iPlayerID = iPlayer.getID()
        
        if not iPlayerID in self.unitHasDiedThisTurn:
            self.unitHasDiedThisTurn[iPlayerID] = {}
        if iUnitID in self.unitHasDiedThisTurn[iPlayerID]:
            if self.unitHasDiedThisTurn[iPlayerID][iUnitID] == True:
                return #Unit can not be saved twice in one turn.

        if pCity:
            newunit = iPlayer.initUnit(iUnit.getUnitType(), pCity.getX(), pCity.getY(), UnitAITypes.NO_UNITAI)
            newunit.setExperience(iUnit.getExperience(), -1)
            newunit.setName(iUnit.getNameNoDesc())
            newunit.setDamage(90, False)
            newunit.setMoves(0)
            newunit.setLevel(iUnit.getLevel())
            self.unitHasDiedThisTurn[iPlayerID][newunit.getID()] = True
            for i in range(gc.getNumPromotionInfos()):
                if iUnit.isHasPromotion(i):
                    newunit.setHasPromotion(i, True)
        
#            iPlayer.createGreatPeople(gc.getInfoTypeForString("UNIT_GREAT_GENERAL"), False, pCity.getX(), pCity,getY())

    def resetGeneralDeath(self, iPlayerID):
        if not iPlayerID in self.unitHasDiedThisTurn:
            self.unitHasDiedThisTurn[iPlayerID] = {}
        else:
            del self.unitHasDiedThisTurn[iPlayerID]
        return
